/******************************************************************************
 * HI 5323 Image Processing
 *
 * fourierDFT.cpp
 ******************************************************************************/

#include <fourierDFT.h>
#include <iostream>
using namespace std;

FourierDFT::FourierDFT(QImage *pInput)
{
    m_pInput = pInput;
    width = m_pInput->width();
    height = m_pInput->height();
    size = width * height;
}

void FourierDFT::oneDDFT(int sign, int width, Complex *line)
{
    Complex sum[width];
    int n, i;
    for(n = 0; n < width; n++)
    {
        sum[n] = Complex(0.0, 0.0);
        for(i = 0; i < width; i++)
            sum[n] += line[i]*euler(sign*2*PI*i*n/width);
    }
    for(n = 0; n < width; n++)
        line[n] = sum[n];

    return;
}

Complex* FourierDFT::twoDDFT()
{
    Complex row[width], colum[height];
    F = new Complex[size];
    int i, j;

    cout << "FourierDFT> 2-D fourier transformation begins:" << endl;

    for(i = 0; i < height; i++)
    {
        uchar *line = m_pInput->scanLine(i);
        for(j = 0; j < width; j++)
            row[j] = Complex(line[j], 0.0);
        // the horizontal transformation
        oneDDFT(-1, width, row);
        // put data back to Farray
        for(j = 0; j < width; j++)
            F[i*width+j] = row[j];
    }

    // rearrange data by colums
    for(i = 0; i < width; i++)
    {
        for(j = 0; j < height; j++)
            colum[j] = F[j*width+i];
        // the veritical transformation
        oneDDFT(-1, height, colum);
        // put data back
        for(j = 0; j < height; j++)
            F[j*height+i] = colum[j]/size;
    }

    return F;
}

Complex *FourierDFT::inverse2DDFT()
{
    Complex row[width], colum[height];
    origArray = new Complex[size];
    int i, j;
    cout << "FourierDFT> Inverse 2-D fourier transformation begins:" << endl;

    for(i = 0; i < height; i++)
    {
        for(j = 0; j < width; j++)
            row[j] = F[i*width+j];
        // the horizontal transformation
        oneDDFT(1, width, row);
        // put data back to origArray
        for(j = 0; j < width; j++)
            origArray[i*width+j] = row[j];
    }

    // rearrange data by colums
    for(i = 0; i < width; i++)
    {
        for(j = 0; j < height; j++)
            colum[j] = origArray[j*width+i];
        // the veritical transformation
        oneDDFT(1, height, colum);
        // put data back
        for(j = 0; j < height; j++)
            origArray[j*height+i] = colum[j];
    }

    return origArray;
}
