/******************************************************************************
 * HI 5323 Image Processing
 *
 * fourierview.h
 ******************************************************************************/

#ifndef __FOURIERVIEW_H
#define __FOURIERVIEW_H

#include <define.h>
#include <complex.h>

#include <qimage.h>
#include <qwidget.h>
#include <qbuttongroup.h>

// Qt3
#ifdef QT3_SUPPORT
#define GroupBox Q3GroupBox
#include <q3groupbox.h>
#define ButtonGroup Q3ButtonGroup
#include <q3buttongroup.h>
#endif

#ifndef QT3_SUPPORT
#define GroupBox QGroupBox
#include <qgroupbox.h>
#define ButtonGroup QButtonGroup
#include <qbuttongroup.h>
#endif


class QPixmap;
class GroupBox;

/**
 *
 */
class FourierView : public QWidget
{
    Q_OBJECT

    enum
    {
        RedBand,
        GrnBand,
        BluBand,
        NoBand
    };

    enum
    {
        ShowNone,
        ShowReal,
        ShowImag,
        ShowAbsReal,
        ShowAbsImag,
        ShowLnReal_1,
        ShowLnImag_1,
        ShowLnAbsReal_1,
        ShowLnAbsImag_1,
        ShowMag,
        ShowMag2,
        ShowLnMag_1,
        ShowLnMag2_1,
        ShowPhase
    };

private:

    Complex* m_pData;
    QImage* m_pImage;

    uint m_iW, m_iH;

    // True if Fourier display is recentered.
    bool m_bRecenter;
    // True if RGB display bands are locked together.
    bool m_bLockRGB;

    QPixmap* m_pFourier;
    QImage* m_pFourierTmp;

    GroupBox *m_pShow;
    ButtonGroup *m_pRed, *m_pGreen, *m_pBlue;

    float m_fMaxMag, m_fMinReal, m_fMaxReal, m_fMinImag, m_fMaxImag;

    void initGui();
    void initData();

    void _convert(int band, int show);

public:

    /**
     * Constructor
     */
    FourierView(Complex *data, QImage* pImage, QWidget *parent=0, const char *name=0);
    ~FourierView();

    uint width() const
    {
        return m_iW;
    };
    uint height() const
    {
        return m_iH;
    };

    void paintEvent(QPaintEvent *);

public slots:

    void setCenter (bool center);
    void setLockRGB(bool lock);

    void redChanged(int show);
    void grnChanged(int show);
    void bluChanged(int show);

signals:

    void pressed();

protected:

    virtual void mousePressEvent(QMouseEvent *);
    virtual void mouseMoveEvent(QMouseEvent *me);

};

#endif
